<?php
class ControllerPaymentTnp extends Controller {

  private $logger;

  public function __construct($arg)
  {
    $this->logger= new Log('tnp.log');
    parent::__construct($arg);
  }

  protected function index() {

    $this->language->load('payment/tnp');
    $this->data['button_confirm'] = $this->language->get('button_confirm');
    $this->data['action'] = "https://biz.traknpay.in/v2/paymentrequest";
  
    $this->load->model('checkout/order');
    $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
    if ($order_info) {
      $api_key = $this->config->get('tnp_api_key');
      $private_salt = $this->config->get('tnp_private_salt');
      $mode=$this->config->get('tnp_mode');
      $name = substr(trim((html_entity_decode($order_info['payment_firstname'] . ' ' . $order_info['payment_lastname'], ENT_QUOTES, 'UTF-8'))), 0, 20);
      $email = substr($order_info['email'], 0, 75);
      $phone = substr(html_entity_decode($order_info['telephone'], ENT_QUOTES, 'UTF-8'), 0, 20);
      $amount = $this->currency->format($order_info['total'], $order_info['currency_code'], false, false);
      $orderid = $this->session->data['order_id'];
      $address_line_1 = html_entity_decode($order_info['payment_address_1'], ENT_QUOTES, 'UTF-8');
      $address_line_2 = html_entity_decode($order_info['payment_address_2'], ENT_QUOTES, 'UTF-8');
      $city = html_entity_decode($order_info['payment_city'], ENT_QUOTES, 'UTF-8');
      $country = html_entity_decode($order_info['payment_iso_code_3'], ENT_QUOTES, 'UTF-8');
      $description = 'TNPV1.4.1_OpencartV1.5: Paying invoice ' . html_entity_decode($order_info['order_id'], ENT_QUOTES, 'UTF-8') . ' of store ' . html_entity_decode($order_info['store_name'], ENT_QUOTES, 'UTF-8') . ' using open cart plugin';
      $state = html_entity_decode($order_info['payment_zone'], ENT_QUOTES, 'UTF-8');
      $zip_code = html_entity_decode($order_info['payment_postcode'], ENT_QUOTES, 'UTF-8');

      if ($order_info['currency_code'] != 'INR') {
        $order_info['currency_code'] = 'INR';
        $currency = html_entity_decode($order_info['currency_code'], ENT_QUOTES, 'UTF-8');
        $amount = $amount / $order_info['currency_value'];
      } else {
        $currency = html_entity_decode($order_info['currency_code'], ENT_QUOTES, 'UTF-8');
      }
      $ip = html_entity_decode($order_info['ip'], ENT_QUOTES, 'UTF-8');
      $return_url = $this->url->link('payment/tnp/callback');

      $data = Array();
      $data['api_key'] = $api_key;
      $data['order_id'] = $orderid;
      $data['currency'] = $currency;
      $data['name'] = $name;
      $data['email'] = $email;
      $data['amount'] = $amount;
      $data['phone'] = $phone;
      $data['address_line_1'] = $address_line_1;
      $data['address_line_2'] = $address_line_2;
      $data['city'] = $city;
      $data['country'] = $country;
      $data['description'] = $description;
      $data['state'] = $state;
      $data['zip_code'] = $zip_code;
      $data['return_url'] = $return_url;
      $data['udf1'] = $ip;
      $data['mode'] = $mode;


      $hash_columns = [
          'address_line_1',
          'address_line_2',
          'amount',
          'api_key',
          'city',
          'country',
          'currency',
          'description',
          'email',
          'mode',
          'name',
          'order_id',
          'phone',
          'return_url',
          'state',
          'udf1',
          'udf2',
          'udf3',
          'udf4',
          'udf5',
          'zip_code',
      ];

      ksort($hash_columns);
      $hash_data = $private_salt;

      foreach ($hash_columns as $column) {
        if (isset($data[$column])) {
          if (strlen($data[$column]) > 0) {
            $hash_data .= '|' . trim($data[$column]);
          }
        }
      }
      $hash = null;
      if (strlen($hash_data) > 0) {
        $hash = strtoupper(hash("sha512", $hash_data));
      }

      $data['hash'] = $hash;

      $this->data = array_merge($this->data, $data);

      if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/tnp.tpl')){
        $this->template = $this->config->get('config_template') . '/template/payment/tnp.tpl';
      } else {
        $this->template = 'default/template/payment/tnp.tpl';
      }
        $this->render();
    }
  }
  
  public function callback(){

    if (isset($this->request->post['order_id'])) {

      $this->load->model('checkout/order');
      $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

      $amount = $this->currency->format($order_info['total'], $order_info['currency_code'], false, false);
      $orderid = $this->session->data['order_id'];

      if ($order_info['currency_code'] != 'INR') {
        $order_info['currency_code'] = 'INR';
        $amount = $amount / $order_info['currency_value'];
      }

      $order_id = $this->request->post['order_id'];
      $response_code = $this->request->post['response_code'];
      $transaction_id = $this->request->post['transaction_id'];
      $response_message = $this->request->post['response_message'];

            $hash_columns=$this->request->post;
            unset($hash_columns['hash']);
            ksort($hash_columns);

            $private_salt = $this->config->get('tnp_private_salt');
            $hash_data = $private_salt;

            foreach ($hash_columns as $key=>$value) {
                if (strlen($value) > 0) {
                    $hash_data .= '|' . $value;
                }
            }
            $hash = null;
            if (strlen($hash_data) > 0) {
                $hash = strtoupper(hash("sha512", $hash_data));
            }

      if ($hash == $this->request->post['hash']) {
        if ($response_code == 0) {
          $this->logger->write("Payment ₹$amount is successful for Order Id $order_id and Transaction Id $transaction_id.");
          $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('tnp_order_status_id'),
              "Payment successful for tnp Order ID: $order_id", true);

          return $this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
        }
        else if ($response_message == 'Transaction Failed') {
          $this->logger->write("Payment unsuccessful for Order Id : $order_id");
               return $this->failed_txn();
        }
        else {
          $this->error($response_message);
        }
      }
      else {
        $this->logger->write("Hash mis matched");
        $this->logger->write("Hash Calculated with data sent by server is : " . $hash_data);
        $this->logger->write("Hash Calculated  : " . $hash);
        $this->logger->write("Hash from the server is : " . $this->request->post['hash']);

        return $this->failure();
      }

    }
    return $this->failed_txn();
  }

  public function failure()
  {
    $this->language->load('payment/tnp');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->data['breadcrumbs'] = array();

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('common/home'),
        'text'      => $this->language->get('text_home'),
        'separator' => false
    );

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('checkout/cart'),
        'text'      => $this->language->get('text_basket'),
        'separator' => $this->language->get('text_separator')
    );

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('checkout/checkout', '', 'SSL'),
        'text'      => $this->language->get('text_checkout'),
        'separator' => $this->language->get('text_separator')
    );

    $this->data['heading_title'] = $this->language->get('heading_title');


    $this->data['text_message']=sprintf($this->language->get('text_message'),$this->url->link('information/contact'));
    $this->data['button_continue'] = $this->language->get('button_continue');

    $this->data['continue'] = $this->url->link('common/home');

    if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/common/success.tpl')) {
      $this->template = $this->config->get('config_template') . '/template/common/success.tpl';
    } else {
      $this->template = 'default/template/common/success.tpl';
    }

    $this->children = array(
        'common/column_left',
        'common/column_right',
        'common/content_top',
        'common/content_bottom',
        'common/footer',
        'common/header'
    );

    $this->response->setOutput($this->render());

  }

  public function failed_txn()
  {
    $this->language->load('payment/tnp');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->data['breadcrumbs'] = array();

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('common/home'),
        'text'      => $this->language->get('text_home'),
        'separator' => false
    );

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('checkout/cart'),
        'text'      => $this->language->get('text_basket'),
        'separator' => $this->language->get('text_separator')
    );

    $this->data['breadcrumbs'][] = array(
        'href'      => $this->url->link('checkout/checkout', '', 'SSL'),
        'text'      => $this->language->get('text_checkout'),
        'separator' => $this->language->get('text_separator')
    );

    $this->data['heading_title'] = 'Failed Payment!';


    $this->data['text_message']=sprintf($this->language->get('text_message_for_failed_txn'),$this->url->link('information/contact'));
    $this->data['button_continue'] = $this->language->get('button_continue');

    $this->data['continue'] = $this->url->link('common/home');

    if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/common/success.tpl')) {
      $this->template = $this->config->get('config_template') . '/template/common/success.tpl';
    } else {
      $this->template = 'default/template/common/success.tpl';
    }

    $this->children = array(
        'common/column_left',
        'common/column_right',
        'common/content_top',
        'common/content_bottom',
        'common/footer',
        'common/header'
    );

    $this->response->setOutput($this->render());
  }
  
  public function error($response_message){
  $this->language->load('payment/tnp');

  $this->document->setTitle($this->language->get('heading_title'));

  $this->data['breadcrumbs'] = array();

  $this->data['breadcrumbs'][] = array(
      'href'      => $this->url->link('common/home'),
      'text'      => $this->language->get('text_home'),
      'separator' => false
  );

  $this->data['breadcrumbs'][] = array(
      'href'      => $this->url->link('checkout/cart'),
      'text'      => $this->language->get('text_basket'),
      'separator' => $this->language->get('text_separator')
  );

  $this->data['breadcrumbs'][] = array(
      'href'      => $this->url->link('checkout/checkout', '', 'SSL'),
      'text'      => $this->language->get('text_checkout'),
      'separator' => $this->language->get('text_separator')
  );

  $this->data['heading_title'] = 'Correct the below error';
  $response_message=substr($response_message,strpos($response_message, ' - ')+2);
  $data['text_message'] = '<h2>'.$response_message.'<h2>';
  $this->data['button_continue'] = $this->language->get('button_continue');

  $this->data['continue'] = $this->url->link('common/home');

  if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/common/success.tpl')) {
    $this->template = $this->config->get('config_template') . '/template/common/success.tpl';
  } else {
    $this->template = 'default/template/common/success.tpl';
  }

  $this->children = array(
      'common/column_left',
      'common/column_right',
      'common/content_top',
      'common/content_bottom',
      'common/footer',
      'common/header'
  );

  $this->response->setOutput($this->render());
}
}
?>